<?php

// $Id: droptor_feed.inc,v 1.2.2.2 2010/08/03 13:37:41 urbanjustin Exp $ 

/**
 * @file
 * The Droptor bridge module
 * Droptor.com - Drupal site organization, monitoring, management and best practices
 *
 * For more information, see:
 * - http://www.droptor.com
 * - http://drupal.org/project/droptor
 */

/**
 * @todo Please document this function.
 * @see http://drupal.org/node/1354
 */
function droptor_feed() {

  /*
   We only show the feed if the hash key is passed in and if the request comes from monrupal.com
   (headers can be spoofed, so we use the hash keys too). It's not paranoia if they're really after you.
   */

  // get hash, domain
  $correct_hash = droptor_check_hash();
  $correct_host = droptor_check_domain();
  $correct_security = droptor_check_connection_security();

  if ( $correct_hash && $correct_host && $correct_security ) {
    $data = array();
    $data['droptor-data-feed']['droptor_feed_version'] = '114';
    $data['droptor-data-feed']['monitor_text_match'] = variable_get('droptor_monitor_text_match', NULL);
    $data['droptor-data-feed']['secure_pages_module_enabled'] = droptor_securepages_module_enabled();
    $data['droptor-data-feed']['ssl_login'] = droptor_ssl_login();
    $data['droptor-data-feed']['ssl_enabled'] = droptor_ssl_enabled();
    $data['droptor-data-feed']['settings_permissions_read_only'] = droptor_settings_permissions_read_only();
    $data['droptor-data-feed']['devel_enabled'] = droptor_devel_module_enabled();
    $data['droptor-data-feed']['extra_files_removed'] = droptor_extra_files_removed();
    $data['droptor-data-feed']['page_cache_enabled'] = droptor_caching_enabled();
    $data['droptor-data-feed']['js_compression_enabled'] = droptor_js_compression_enabled();
    $data['droptor-data-feed']['css_compression_enabled'] = droptor_css_compression_enabled();
    $data['droptor-data-feed']['page_compression_enabled'] = droptor_page_compression_enabled();
    $data['droptor-data-feed']['theme_registry_rebuild_enabled'] = droptor_theme_registry_rebuild_enabled();
    $data['droptor-data-feed']['last_cron'] = droptor_last_cron();
    $data['droptor-data-feed']['node_statistics'] = droptor_node_statistics();
    $data['droptor-data-feed']['user_statistics'] = droptor_user_statistics();
    $data['droptor-data-feed']['statistics'] = droptor_statistics();
    $data['droptor-data-feed']['last_nodes'] = droptor_last_nodes();
    $data['droptor-data-feed']['last_users'] = droptor_last_users();
    $data['droptor-data-feed']['watchdog_size'] = droptor_watchdog_size();
    $data['droptor-data-feed']['site_name'] = variable_get('site_name', '{unknown}');
    $data['droptor-data-feed']['modules'] = droptor_modules();
    $data['droptor-data-feed']['module_update_data'] = droptor_module_update_data();
    $data['droptor-data-feed']['variables_data'] = droptor_variables_data();
    $data['droptor-data-feed']['recent_pages_not_found'] = droptor_recent_pages_not_found();
    $data['droptor-data-feed']['error_level'] = droptor_error_level();
    $data['droptor-data-feed']['last_logins'] = droptor_last_logins();
    $data['droptor-data-feed']['feed_over_ssl'] = variable_get('droptor_ssl_feed', 0);

    if (variable_get('droptor_debug_mode', 0) == 1) {
      $data['droptor-data-feed']['debug_remote_addr'] = $_SERVER['REMOTE_ADDR'];
    }

    //droptor_pr($data);
	echo serialize($data);
  }
  elseif (!$correct_hash) {
    echo t('Invalid hash');
  }
  elseif (!$correct_host) {
    echo t('Invalid host') . '  (' . $_SERVER['REMOTE_ADDR'] . ')';
  }
  elseif (!$correct_security) {
    echo t('Invalid security');
  }

  exit; // http://stackoverflow.com/questions/2102688/how-do-you-replace-the-page-tpl-php-that-loads-from-a-module-in-drupal
}

/**
 * Grabs the failed logins in the last hour
 *
 * @param n/a
 *
 * @return
 *   An array of who and when of failed logins
 */
function droptor_last_logins() {
  $results = db_query('SELECT message, variables, timestamp FROM {watchdog} WHERE message LIKE :message AND now()-timestamp > :lasthour ORDER BY timestamp DESC', array(':message'=>'login attempt failed%',':lasthour' => 60 * 60));
  $last_logins = array();

  foreach ($results as $result) {
    $variables = unserialize($result->variables);
    $last_logins['failed'][$result->timestamp] = $variables['%user'];
  }
  return $last_logins;
}

/**
 * Returns error level value
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   The error_level var
 */
function droptor_error_level() {
  return variable_get('error_level', 1);
}

/**
 * Grabs all system variables
 *
 * @param n/a
 *
 * @return
 *   An array of all of the variables for the site
 */
function droptor_variables_data() {
  $variables = array();
  $results = db_query('SELECT name, value FROM {variable} ORDER BY name');

  foreach ($results as $result) {
    $variables[$result->name] = unserialize($result->value);
  }
  return serialize($variables);
}

/**
 * Grabs the last 404 errors in logs, excluding the checks we run for drupal files like README
 *
 * @param n/a
 *
 * @return
 *   An array of the pages not found and some associated data
 */
function droptor_recent_pages_not_found() {
  $data = array();
  $row = 0;

  $results = db_query_range('SELECT message, hostname, referer, timestamp FROM {watchdog} WHERE type = :type AND message NOT IN(:list) ORDER BY timestamp DESC', 
	0, 10, array(':type'=>'page not found',':list'=>'"UPGRADE.txt", "MAINTAINERS.txt", "README.txt", "INSTALL.pgsql.txt", "INSTALL.txt", "LICENSE.txt", "INSTALL.mysql.txt", "COPYRIGHT.txt", "CHANGELOG.txt"'));

  foreach ($results as $result) {
    $data[$row]['message'] = $result->message;
    $data[$row]['hostname'] = $result->hostname;
    $data[$row]['referer'] = $result->referer;
    $data[$row]['timestamp'] = $result->timestamp;

    $row++;
  }

  return $data;
}

/**
 * Grabs all system variables
 *
 * @param n/a
 *
 * @return
 *   An array of all of the variables for the site
 */
function droptor_watchdog_size() {
  return db_query('SELECT COUNT(*) FROM {watchdog}')->fetchField();
}

/**
 * Get total nodes created by day, for the last 30 days, for charting
 *
 * @param n/a
 *
 * @return
 *   An array of total nodes created each day for the last month
 */
function droptor_node_statistics() {
  $results = db_query_range("SELECT COUNT(*) AS total, DATE_FORMAT(FROM_UNIXTIME(created), '%m/%%d/%Y') AS dt FROM {node}  GROUP BY DATE_FORMAT(FROM_UNIXTIME(created), '%m/%%d/%Y') ORDER BY created DESC", 0, 30);
  $data = array();
  foreach ($results as $result) {
    $data[$result->dt] = $result->total;
  }

  return $data;
}

/**
 * Get total users created by day, for the last 30 days, for charting
 *
 * @param n/a
 *
 * @return
 *   An array of total users created each day for the last month
 */
function droptor_user_statistics() {
  $results = db_query_range("SELECT COUNT(*) AS total, DATE_FORMAT(FROM_UNIXTIME(created), '%m/%%d/%Y') AS dt FROM {users}  GROUP BY DATE_FORMAT(FROM_UNIXTIME(created), '%m/%%d/%Y') ORDER BY created DESC", 0, 30);
  $data = array();
  foreach ($results as $result) {
    $data[$result->dt] = $result->total;
  }

  return $data;
}

/**
 * Get lsat 15 nodes created
 *
 * @param n/a
 *
 * @return
 *   An array of the details of last 15 nodes created
 */
function droptor_last_nodes() {
  $last_five_nodes = array();
  $query = db_query_range('SELECT title, nid, created FROM {node} ORDER BY created DESC', 0, 15);

  $count = 0;
  foreach ($query as $data) {
    $last_five_nodes[$count]['url'] = drupal_get_path_alias('node/' . $data->nid);
    $last_five_nodes[$count]['title'] = $data->title;
    $last_five_nodes[$count]['created'] = $data->created;
    $count++;
  }

  return $last_five_nodes;
}

/**
 * Get lsat 15 users created
 *
 * @param n/a
 *
 * @return
 *   An array of the details of last 15 users created
 */
function droptor_last_users() {
  $last_five_users = array();
  $query = db_query_range('SELECT uid, mail, name, created FROM {users} ORDER BY created DESC', 0, 15);

  $count = 0;
  foreach ($query as $data) {
    $last_five_users[$count]['uid'] = $data->uid;
    $last_five_users[$count]['name'] = $data->name;
    $last_five_users[$count]['email'] = $data->mail;
    $last_five_users[$count]['created'] = $data->created;
    $count++;
  }

  return $last_five_users;
}

/**
 * Wrapper function to get node and user stats
 *
 * @param n/a
 *
 * @return
 *   An array of various node and user statistics
 */
function droptor_statistics() {
  $stats = array();

  // node  and user creations over the last month
  $stats = droptor_node_and_user_time_stats();
  $stats = array_merge($stats, droptor_node_and_user_totals());

  return $stats;
}

/**
 * Get the last time cron ran
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   The last time cron ran in std php date format
 */
function droptor_last_cron() {
  $last_cron = variable_get('cron_last', NULL);

  return $last_cron ? date('Y-m-d H:i:s e', $last_cron) : FALSE;
}

/**
 * Determine if the theme registry is being rebuilt
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   An array of the details of last 15 nodes created
 */
function droptor_theme_registry_rebuild_enabled() {
  return variable_get('devel_rebuild_theme_registry', FALSE) == 1 ? 1 : 0;
}

/**
 * Check performance setting
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   On or off state for this performance setting
 */
function droptor_js_compression_enabled() {
  return variable_get('preprocess_js', FALSE) == 1 ? 1 : 0;
}

/**
 * Check performance setting
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   On or off state for this performance setting
 */
function droptor_page_compression_enabled() {
  return variable_get('page_compression', FALSE) == 1 ? 1 : 0;
}

/**
 * Check performance setting
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   On or off state for this performance setting
 */
function droptor_css_compression_enabled() {
  return variable_get('preprocess_css', FALSE) == 1 ? 1 : 0;
}

/**
 * Check performance setting
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   On or off state for this performance setting
 */
function droptor_caching_enabled() {
  return variable_get('cache', FALSE) == 1 ? 1 : 0;
}

/**
 * Get the full module update status data for all modules
 *
 * @param n/a
 *
 * @return
 *   A serialized dump of the module update array data
 */
function droptor_module_update_data() {
  $module_meta_data = update_get_available(TRUE);
  return serialize(update_calculate_project_data($module_meta_data));
}

/**
 * A list of every module in the system
 *
 * @param n/a
 *
 * @return
 *   A serialized list of module details array
 */
function droptor_modules() {
  $query_result = db_query('SELECT name, status, info, type FROM {system} ORDER BY status DESC, name');
  $modules = array();

  foreach ($query_result as $data) {
    $info = unserialize($data->info);
    $modules[$data->name]['version'] = $info['version'];
    $modules[$data->name]['status'] = $data->status;
    $modules[$data->name]['name'] = $info['name'];
    $modules[$data->name]['type'] = $data->type;
  }

  return serialize($modules);
}

/**
 * Check to see if the unneeded release files with Drupal are removed
 *
 * @param n/a
 *
 * @return
 *   True if they are removed, false if they aren't
 */
function droptor_extra_files_removed() {
  $clean_codebase = TRUE;
  $url = url(NULL, array('absolute' => TRUE));
  $files_to_remove = array('CHANGELOG.txt', 'COPYRIGHT.txt', 'INSTALL.mysql.txt', 'INSTALL.pgsql.txt', 'INSTALL.txt', 'LICENSE.txt', 
    'MAINTAINERS.txt', 'README.txt', 'UPGRADE.txt');

  foreach ($files_to_remove as $file) {
    if (droptor_fast_url_check($url . $file)) {
      $clean_codebase = FALSE;
    }
  }

  return $clean_codebase ? 1 : 0;
}

/**
 * Check if devel is on
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   True if on; false otherwise
 */
function droptor_devel_module_enabled() {
  return droptor_is_module_installed_and_enabled('devel');
}

/**
 * Determines if settings.php is read-only
 *
 * @param n/a
 *
 * @return
 *   TRUE if the settings file is read-only, and FALSE if it isn't.
 */
function droptor_settings_permissions_read_only() {
  $settings_permissions_read_only = TRUE;
  $writes = array('2', '3', '6', '7'); // http://en.wikipedia.org/wiki/File_system_permissions
  $settings_file = './' . conf_path(FALSE, TRUE) . '/settings.php';
  $permissions = drupal_substr(sprintf('%o', fileperms($settings_file)), -4);

  foreach ($writes as $bit) {
    if (strpos($permissions, $bit)) {
      $settings_permissions_read_only = FALSE;
      break;
    }
  }

  return $settings_permissions_read_only;

}

/**
 * Determines if secure pages module is enabled.
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   TRUE if the secure pages module is enabled, and FALSE if it isn't.
 */
function droptor_securepages_module_enabled() {
  // secure pages module is installed and enabled
  return droptor_is_module_installed_and_enabled('securepages') && variable_get('securepages_enable', FALSE) ? TRUE : FALSE;
}

/**
 * Checks to see if SSL login is required
 *
 * @param n/a
 *
 * @return
 *   True if login is secure, false if not
 */
function droptor_ssl_login() {
  $login_safe = 0;
  $securepages_login_query = db_query('SELECT value FROM {variable}  WHERE name = :name', array(':name' => 'securepages_pages'));
  foreach ($securepages_login_query as $securepages_login_results) {
    $pages = $securepages_login_results->value;
    if (strpos($pages, 'user*') > 0) {
      $login_safe = 1;
    } // to do - check the ignore pages field      
  }

  return $login_safe;
}

/**
 * Gathers various user and node stats for various time ranges
 * @todo - Total stats revamp in feed version 2
 *
 * @param n/a
 *
 * @return
 *   An array of data for various activity data over various recent time frames
 */
function droptor_node_and_user_time_stats() {
  $thirty_days_ago_in_seconds = REQUEST_TIME -(30 * 24 * 60 * 60);
  $seven_days_ago_in_seconds = REQUEST_TIME -(7 * 24 * 60 * 60);
  $twenty_four_hours_ago_in_seconds = REQUEST_TIME -(24 * 60 * 60);

  $stats = array();
  $stats['nodes_in_the_last_30_days'] = 0;
  $stats['nodes_in_the_last_7_days'] = 0;
  $stats['nodes_in_the_last_24_hours'] = 0;
  $stats['users_in_the_last_30_days'] = 0;
  $stats['users_in_the_last_7_days'] = 0;
  $stats['users_in_the_last_24_hours'] = 0;

  $nodes_result = db_query('SELECT created FROM {node}  WHERE created > :created', array(':created' => $thirty_days_ago_in_seconds));

  foreach ($nodes_result as $node) {
    if ($node->created > $twenty_four_hours_ago_in_seconds) {
      $stats['nodes_in_the_last_24_hours']++;
    }

    if ($node->created > $seven_days_ago_in_seconds) {
      $stats['nodes_in_the_last_7_days']++;
    }

    if ($node->created > $thirty_days_ago_in_seconds) {
      $stats['nodes_in_the_last_30_days']++;
    }
  }

  $users_result = db_query('SELECT created FROM {users}  WHERE created > :created', array(':created' => $thirty_days_ago_in_seconds));

  foreach ($users_result as $user) {
    if ($user->created > $twenty_four_hours_ago_in_seconds) {
      $stats['users_in_the_last_24_hours']++;
    }

    if ($user->created > $seven_days_ago_in_seconds) {
      $stats['users_in_the_last_7_days']++;
    }

    if ($user->created > $thirty_days_ago_in_seconds) {
      $stats['users_in_the_last_30_days']++;
    }
  }

  return $stats;
}

/* INTERNALS */
/* INTERNALS */
/* INTERNALS */

/**
 * Helper function to see if module is enabled
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   True if the module is enabled and false otherwise
 */
function droptor_is_module_installed_and_enabled($name) {
  $module_query = db_query('SELECT count(*) as module_installed FROM {system} WHERE name = :name and status = :status', array(':name' => $name, ':status' => 1));

  foreach ($module_query as $module_results) {
    return $module_results->module_installed;
  }
}

/**
 * Quickly check if a URL exists
 * b/c drupal_http_request() is wicked slow compared to straight curl
 *
 * @param n/a
 *
 * @return
 *   True if the URL exists, false otherwise
 */
function droptor_fast_url_check($url) {
  $valid = FALSE;

  if (in_array('curl', get_loaded_extensions())) {
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($curl, CURLOPT_HEADER, FALSE);
    curl_exec($curl);
    $result = curl_getinfo($curl);
    curl_close($curl);
    $valid = $result['http_code'] == '200' ? TRUE : FALSE;
  }
  else {
    $result = drupal_http_request($url);
    $valid = $result->code == 200 ? TRUE : FALSE;
  }

  return $valid;
}

/**
 * Total number of nodes (and by type) and total user count
 * Deprecated in feed version 2
 *
 * @param n/a
 *
 * @return
 *   An array of user and node stats
 */
function droptor_node_and_user_totals() {
  $stats = array();

  // total users
  $total_users = db_query('SELECT COUNT(*) as total_users FROM {users}')->fetchField();
  $stats = is_numeric($total_users) ? array_merge($stats, array('total_users'=>$total_users)) : $stats;

  // total nodes
  $total_nodes_query = db_query('SELECT COUNT(*) as total_nodes, nt.name as type FROM {node} n JOIN {node_type} nt ON n.type = nt.type GROUP BY type ORDER BY nt.name');
  $total_nodes = 0;
  $nodes_by_type = array();
  
  foreach ($total_nodes_query as $node_type_data) {
    $nodes_by_type[$node_type_data->type] = $node_type_data->total_nodes;
    $total_nodes += $node_type_data->total_nodes;
  }

  $stats['total_nodes'] = $total_nodes;
  $stats['total_nodes_by_type'] = $nodes_by_type;

  return $stats;
}